import datetime
import os
import subprocess

from django.conf import settings
from django.core.files.storage import default_storage
from django.core.mail import send_mail
from django.core.management.base import BaseCommand
from django.template import loader
from django.utils import translation
from django.utils.timezone import now
from django.utils.translation import ugettext_lazy as _

from panikdb.regie.models import PigeEmailRequest


class Command(BaseCommand):
    def handle(self, verbosity, **kwargs):
        translation.activate(settings.LANGUAGE_CODE)
        pige_path = default_storage.path('pige')
        for email_request in PigeEmailRequest.objects.filter(status=''):
            download_path = os.path.join(pige_path, email_request.get_email_download_spec())
            if not os.path.exists(download_path):
                proc = subprocess.run(
                    [
                        'curl',
                        '--fail',
                        '--silent',
                        '--output',
                        download_path,
                        settings.PIGE_DOWNLOAD_BASE_URL + '/' + email_request.get_email_download_spec(),
                    ]
                )
                email_request.download_timestamp = now()
                if proc.returncode != 0:
                    email_request.status = 'download-error (%s)' % proc.returncode
                else:
                    email_request.status = 'downloaded'
            else:
                email_request.download_timestamp = now()
                email_request.status = 'downloaded'  # was already there
            email_request.save()

        for email_request in PigeEmailRequest.objects.filter(
            status='downloaded', download_timestamp__lt=now() - datetime.timedelta(minutes=10)
        ):
            subject = '%s - %s' % (settings.SITE_TITLE, _('record download'))
            body = loader.get_template('regie-pige-email.txt')
            context = {
                'email_request': email_request,
            }
            try:
                send_mail(
                    subject, body.render(context), from_email=None, recipient_list=[email_request.email]
                )
                email_request.status = 'sent'
            except Exception as e:
                print(e)
                email_request.status = 'sent-error'
            email_request.save()

        for email_request in PigeEmailRequest.objects.filter(
            status='sent', download_timestamp__lt=now() - datetime.timedelta(days=5)
        ):
            download_path = os.path.join(pige_path, email_request.get_email_download_spec())
            if os.path.exists(download_path):
                os.unlink(download_path)
            email_request.delete()
